{-----------------------------------------------------------------------------
   Unit Name: edittime
   Purpose  : Unit for editing FollowTimeItem
   Author   : Vesa Lappalainen
   Date     : 27.12.1997
   Changed  :
   Todo     :
-----------------------------------------------------------------------------}

unit edittime;

interface

uses
  SysUtils, Classes,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QStdCtrls, QExtCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
{$endif}
  follitem, filecomb, savepos;

type
  TEditTimeItem = class(TForm)
    PanelBase: TPanel;
    PanelButtons: TPanel;
    PanelButtonsInside: TPanel;
    ButtonOK: TButton;
    ButtonCancel: TButton;
    Panel1: TPanel;
    Label1: TLabel;
    EditStart: TEdit;
    Panel2: TPanel;
    Label2: TLabel;
    EditStop: TEdit;
    Panel3: TPanel;
    Label3: TLabel;
    EditDiff: TEdit;
    Panel4: TPanel;
    Label4: TLabel;
    EditExpl: TEdit;
    ButtonCalcStart: TButton;
    ButtonCalcStop: TButton;
    Panel5: TPanel;
    Label5: TLabel;
    CBProjects: TComboBox;
    SavePos1: TSavePos;
    ButtonContinue: TButton;
    ButtonDateStart: TButton;
    ButtonDateStop: TButton;
    procedure EditStartChange(Sender: TObject);
    procedure EditStopChange(Sender: TObject);
    procedure ButtonCalcStartClick(Sender: TObject);
    procedure ButtonCalcStopClick(Sender: TObject);
    procedure CBProjectsChange(Sender: TObject);
    procedure ButtonDateStopClick(Sender: TObject);
    procedure ButtonDateStartClick(Sender: TObject);
  private
    { Private declarations }
    item : TFollowTimeItem;
    projects : TFileCombo;
  public
    { Public declarations }
    function Edit(prj:TFileCombo;aitem:TFollowTimeItem;const text:string;cont:boolean=false):integer;
  end;

function DiffStrToTime(const s:string):TDateTime;
function TimeToDiffStr(d:TDateTime):string;

var
  EditTimeItem: TEditTimeItem;

implementation
uses askdate,kdouble;
{$ifdef CLX}
{$R *.xfm}
{$else}
{$R *.dfm}
{$endif}


function DiffStrToTime(const s:string):TDateTime;
var st:string; p : integer; d : double; sign:integer;
begin
  sign := 1;
  d := 0;
  st := trim(s);
  if ( Length(st) > 0 ) and ( st[1] = '-' ) then begin
    sign := -1; Delete(st,1,1);
  end;
  p := pos(' ',st);
  if ( p > 0 ) then begin
    d := StrToFloatDef(copy(st,1,p-1),0);
    Delete(st,1,p);
  end;
  Result := sign*(d + StrToTime(st));
end;

function TimeToDiffStr(d:TDateTime):string;
var s:string;
begin
  s := '';
  if ( d < 0 ) then begin s :='-'; d := -d; end;
  if ( d >= 1 ) then s := s + IntToStr(trunc(d)) + ' ';
  Result := s + TimeToStr(d);
end;

function TEditTimeItem.Edit(prj:TFileCombo;aitem:TFollowTimeItem;
                            const text:string;cont:boolean):integer;
begin
  ButtonOK.Align := alRight;
  Result := mrCancel;
  item := aitem;
  projects := prj;
  if ( item = nil ) then exit;
  Caption := text;
  if (prj <> nil ) and ( prj.cb <> nil ) then begin
    CBProjects.Items := prj.cb.Items;
  end;
  CBProjects.ItemIndex := prj.Find(item);
  EditStart.Text := DateTimeToStr(item.DStart);
  EditStop.Text := DateTimeToStr(item.DStop);
  EditDiff.Text := TimeToDiffStr(item.DDiff);
  EditExpl.Text := Item.Expl;
  if ( cont ) then begin
    ButtonContinue.Visible := true;
    ButtonCancel.Caption := 'Stop no &add';
  end else begin
    ButtonContinue.Visible := false;
    ButtonCancel.Caption := 'C&ancel';
  end;

  Result := ShowModal;
  if ( Result = mrCancel ) then exit;
  item.DStart := StrToDateTime(EditStart.Text);
  item.DStop := StrToDateTime(EditStop.Text);
  item.Expl  := EditExpl.Text;
  if ( projects = nil ) then exit;
  item.GetId(projects.Obj[CBProjects.ItemIndex] as TFollowProjectItem);
end;

procedure TEditTimeItem.EditStartChange(Sender: TObject);
begin
  if ( item = nil ) then exit;
  try
    item.DStart := StrToDateTime(EditStart.Text);
  except
  end;
  EditDiff.Text := TimeToDiffStr(item.DDiff);
end;

procedure TEditTimeItem.EditStopChange(Sender: TObject);
begin
  if ( item = nil ) then exit;
  try
    item.DStop := StrToDateTime(EditStop.Text);
  except
  end;
  EditDiff.Text := TimeToDiffStr(item.DDiff);
end;

procedure TEditTimeItem.ButtonCalcStartClick(Sender: TObject);
begin
  if ( item = nil ) then exit;
  try
    item.DStart := item.DStop - DiffStrToTime(EditDiff.Text);
  except
  end;
  EditStart.Text := DateTimeToStr(item.DStart);
end;

procedure TEditTimeItem.ButtonCalcStopClick(Sender: TObject);
begin
  if ( item = nil ) then exit;
  try
    item.DStop := item.DStart + DiffStrToTime(EditDiff.Text);
  except
  end;
  EditStop.Text := DateTimeToStr(item.DStop);
end;

procedure TEditTimeItem.CBProjectsChange(Sender: TObject);
begin
  if ( projects = nil ) then exit;
  Hint := projects.Obj[CBProjects.ItemIndex].HintStr;
end;

function EditDate(const q:string;d:TDateTime):TDateTime;
var t,od,nd:TDateTime;
begin
  Result := 0;
  t := StrToTime(TimeToStr(d));
  od := trunc(d);
  if not DoAskDate(q,nd,od) then exit;
  if ( nd = od ) then exit;
  Result := nd + t;
end;

procedure TEditTimeItem.ButtonDateStartClick(Sender: TObject);
var d: TDateTime;
begin
  if ( item = nil ) then exit;
  d := EditDate('Give start date',item.DStart);
  if ( d = 0 ) then exit;
  item.DStart := d;
  EditStart.Text := DateTimeToStr(item.DStart);
end;

procedure TEditTimeItem.ButtonDateStopClick(Sender: TObject);
var d: TDateTime;
begin
  if ( item = nil ) then exit;
  d := EditDate('Give stop date',item.DStop);
  if ( d = 0 ) then exit;
  item.DStop := d;
  EditStop.Text := DateTimeToStr(item.DStop);
end;

end.
