{-----------------------------------------------------------------------------
   Unit Name: fileobj
   Purpose  :   Unit for simple text file input and output using object
                representative for every line in file
   Author   : Vesa Lappalainen
   Date     : 26.12.1997
   Changes  : 27.12.1997/vl
   Todo     :
-----------------------------------------------------------------------------}

unit fileobj;

interface
  uses Classes;

type

  TFileObject = class
  public
    constructor Create(s:string);
    procedure SetStr(const s:string); virtual;
    function GetStr:string; virtual;
    function GetShowStr:string; virtual;
    function GetHintStr:string; virtual;
    property Str:string read GetStr write SetStr;
    property ShowStr : string read GetShowStr;
    property HintStr : string read GetHintStr;
    procedure SetSt(const st:TStrings); virtual;
    procedure GetSt(st:TStrings); virtual;
    function Same(o:TFileObject):boolean; virtual;
    function Compare(o:TFileObject;i:integer):integer; virtual;
    class function ItemCount : integer; virtual;
    function GetItem(i:integer):string; virtual;
    procedure SetItem(i:integer; const s:string); virtual;
    class function GetHead(i:integer):string; virtual;
    property Items[i:integer]:string read GetItem write SetItem;
    property Head[i:integer]:string read GetHead;
  end;

  TFileObjectType = class of TFileObject;
  TFileObjects = class;
  TItemAddEvent = function (sender:TFileObjects;o:TFileObject) : boolean  of object;

  TFileObjects = class
  private
    FType : TFileObjectType;
    FName : string;
    FItems : TList;
    FItem,FDummy : TFileObject;
    FOnAdd : TItemAddEvent;
    function AssignName(var f:Text; const n:string):boolean;
    function GetObject(i:integer):TFileObject; virtual;
    function GetCount : integer; virtual;
  protected
    property Items : Tlist read FItems;
    function AddSNU(const s:string):integer; virtual;
    function AddNU(o:TFileObject):integer; virtual;
  public
    constructor Create2(AType:TFileObjectType;const AName:string); virtual;
    destructor Destroy; override;
    procedure Clear; virtual;
    function ReadFromFile(const name:string):boolean; virtual;
    function WriteToFile(const name:string):boolean; virtual;
    function AppendToFile(o:TFileObject; const name:string):boolean; virtual;
    function Add(o:TFileObject):integer; virtual;
    function AddS(const s:string):integer; virtual;
    function InformNew(o:TFileObject):boolean; virtual;
    function Update:boolean; virtual;
    function Find(o:TFileObject) : integer; virtual;
    property Obj[i:integer]:TFileObject read GetObject;
    property Dummy : TFileObject read FDummy;
    property Item : TFileObject read FItem;
    property Count : integer read GetCount;
    property IType : TFileObjectType read FType;
    property Name : string read FName;
    property OnAdd : TItemAddEvent read FOnAdd write FOnAdd;
  end;

implementation
uses SysUtils;

//------------------------------------------------------------------------------
constructor TFileObject.Create(s:string);
begin
  inherited Create;
  str := s;
end;

procedure TFileObject.SetSt(const st:TStrings);
begin
end;

procedure TFileObject.GetSt(st:TStrings);
begin
end;

procedure TFileObject.SetStr(const s:string);
var st:TStrings;
begin
  st := TStringList.Create;
  st.CommaText := s + ',,,,,,,,,,,,,,,,,,,,,,,,,,,,,,';
  SetSt(st);
  st.Free;
end;

function TFileObject.GetStr:string;
var st:TStrings;
begin
  st := TStringList.Create;
  GetSt(st);
  Result := st.CommaText;
  st.Free;
end;

function TFileObject.GetShowStr:string; begin Result := GetStr; end;
function TFileObject.GetHintStr:string; begin Result := GetStr; end;

function TFileObject.Same(o:TFileObject):boolean;
begin
  Result := (str = o.Str);
end;

class function TFileObject.ItemCount : integer;
begin
  Result := 1;
end;

function TFileObject.GetItem(i:integer):string;
var st:TStrings;
begin
  Result := '';
  if ( i < 0 ) then exit;
  st := TStringList.Create;
  GetSt(st);
  if ( i < st.Count ) then
    Result := st[i];
  st.Free;
end;

procedure TFileObject.SetItem(i:integer; const s:string);
var st:TStrings;
begin
  if ( i < 0 ) then exit;
  st := TStringList.Create;
  GetSt(st);
  if ( i < st.Count ) then begin
    st[i] := s;
    str := st.CommaText;
  end;
  st.Free;
end;

class function TFileObject.GetHead(i:integer):string;
begin
  Result := IntToStr(i);
end;

function TFileObject.Compare(o:TFileObject;i:integer):integer;
begin
  Result := CompareText(Items[i],o.Items[i]);
  if ( Result = 0 ) then Result := CompareText(Str,o.Str);
end;

//------------------------------------------------------------------------------
// TFileObjects
//------------------------------------------------------------------------------
constructor TFileObjects.Create2(AType:TFileObjectType;const AName:string);
begin
  FType  := AType;
  FName  := AName;
  FItems := TList.Create;
  FDummy := AType.Create('');
  FItem  := AType.Create('');
  inherited Create;
end;

procedure TFileObjects.Clear;
var i:integer; o : TObject;
begin
  if items = nil then exit;
  for i:=0 to items.Count-1 do begin
    o := items[i];
    o.Free;
  end;
  items.Clear;
  Update;
end;

destructor TFileObjects.Destroy;
begin
  if items <> nil then begin
    Clear;
    items.Free;
  end;
  FItem.Free;
  FDummy.Free;
  inherited;
end;

function TFileObjects.AssignName(var f:Text; const n:string):boolean;
var name : string;
begin
  name := n;
  if n = '' then name := FName;
  AssignFile(f,name);
  Result := false;
end;

function TFileObjects.ReadFromFile(const name:string):boolean;
var f : Text; s:string;
begin
  Result := AssignName(f,name);
  if items = nil then exit;
{$I-}
  Reset(f);
  if ( IOResult <> 0 ) then Exit;
{$I+}
  while not eof(f) do begin
    Readln(f,s);
    if ( s <> '' ) then AddSNU(s);
  end;
  Close(f);
  Result := true;
  Update;
end;

function TFileObjects.WriteToFile(const name:string):boolean;
var f : Text;
    i:integer; o : TFileObject;
begin
  Result := AssignName(f,name);
  if items = nil then exit;
  Rewrite(f);
  for i:=0 to items.Count-1 do begin
    o := items[i];
    Writeln(f,o.Str);
  end;
  Close(f);
  Result := true;
end;

function TFileObjects.AppendToFile(o:TFileObject; const name:string):boolean;
var f : TextFile;
begin
//  Result := 
  AssignName(f,name);
{$I-}
  Append(f);
  if ( IOResult <> 0 ) then Rewrite(f);
{$I+}
  Writeln(f,o.str);
  Close(f);
  Result := true;
end;

function TFileObjects.InformNew(o:TFileObject):boolean;
begin
  Result := true;
end;

function TFileObjects.Update:boolean;
begin
  Result := true;
end;

function TFileObjects.GetObject(i:integer):TFileObject;
begin
  if ( i < 0 ) or ( items.Count <= i ) then Result := Dummy
  else
    Result := items[i];
end;

function TFileObjects.AddNU(o:TFileObject):integer;
begin
  Result := -1;
  if items = nil then exit;
  if ( Assigned(OnAdd) and ( not ( OnAdd(self,o) ) ) ) then exit;
  if ( InformNew(o) ) then items.Add(o) else Exit;
  Result := items.Count-1;
//  Update;
end;

function TFileObjects.AddSNU(const s:string):integer;
var o:TFileObject;
begin
  Result := -1;
  if items = nil then exit;
  o := FType.Create(s);
  if ( AddNU(o) < 0 ) then o.Free;
end;

function TFileObjects.AddS(const s:string):integer;
begin
  Result := AddSNU(s);
  Update;
end;

function TFileObjects.Add(o:TFileObject):integer;
begin
  Result := AddNU(o);
  if ( Result < 0 ) then exit;
  Update;
end;

function TFileObjects.GetCount : integer;
begin
  Result := 0;
  if items = nil then exit;
  Result := Items.Count;
end;


function TFileObjects.Find(o:TFileObject) : integer;
var i:integer;
begin
  Result := -1;
  for i:=0 to Count do
    if Obj[i].Same(o) then begin Result := i; exit; end;
end;

end.
