{-----------------------------------------------------------------------------
   Unit Name: follitem
   Purpose  : Unit to handle TimeFollow's data strcutures
   Author   : Vesa Lappalainen
   Date     : 26.12.1997
   Changes  : 27.12.1997/vl
              + explanation field to time file
   Todo     :
-----------------------------------------------------------------------------}

unit follitem;

interface
uses sysutils,classes,fileobj;

type
  TFollowProjectItem = class(TFileObject)
    MainId : string;
    SubId : string;
    Expl  : string;
    Level : integer;
    Prog  : string;
  public
    function Same(o:TFileObject):boolean; override;
    procedure SetSt(const st:TStrings); override;
    procedure GetSt(st:TStrings); override;
    function GetHintStr:string; override;
    function GetShowStr:string; override;
  end;

  TFollowTimeItem = class(TFileObject)
  private
    FDStart : TDateTime;
    FDStop  : TDateTime;
    FDDiff  : TDateTime;
    FStart : string;
    FStop : string;
    FDiff : string;
  public
    MainId : string;
    SubId : string;
    Expl : string;
    procedure SetSt(const st:TStrings); override;
    procedure GetSt(st:TStrings); override;
    function Same(o:TFileObject):boolean; override;
    class function ItemCount : integer;  override;
    function GetItem(i:integer):string; override;
    procedure SetItem(i:integer; const s:string); override;
    class function GetHead(i:integer):string; override;
    function Compare(o:TFileObject;i:integer):integer; override;

    procedure GetId(const prj:TFollowProjectItem); virtual;
    procedure DoStart; virtual;
    procedure DoStop; virtual;
    procedure DoStopExpl(const exp:string); virtual;
    procedure SetDStart(d:TDateTime); virtual;
    procedure SetDStop(d:TDateTime); virtual;
    procedure SetDDiff(d:TDateTime); virtual;
    procedure SetStart(const s:string); virtual;
    procedure SetStop(const s:string); virtual;

    property DStart : TDateTime read FDStart write SetDStart;
    property DStop  : TDateTime read FDStop write SetDStop;
    property DDiff  : TDateTime read FDDiff write SetDDiff;
    property Start : string read FStart write SetStart;
    property Stop : string read FStop write SetStop;
    property Diff : string read FDiff;
  end;



implementation
uses kdouble,edittime;

//------------------------------------------------------------------------------
procedure TFollowProjectItem.SetSt(const st:TStrings);
begin
  MainId := st[0];
  SubId := st[1];
  Expl := st[2];
  Level := StrToIntDef(st[3],1);
  Prog := st[4];
end;

procedure TFollowProjectItem.GetSt(st:TStrings);
begin
  st.Add(MainId);
  st.Add(SubId);
  st.Add(Expl);
  st.Add(IntToStr(Level));
  st.Add(Prog);
end;

function TFollowProjectItem.GetHintStr:string;
begin
  Result := MainId + ' ' + SubId + ' ' + Expl;
end;

function TFollowProjectItem.GetShowStr:string;
begin
  Result := Expl;
end;

function TFollowProjectItem.Same(o:TFileObject):boolean;
var pi : TFollowTimeItem;
begin
  if ( o is TFollowTimeItem ) then begin
    pi := o as TFollowTimeItem;
    Result := ( pi.MainId = MainId ) and ( pi.SubId = SubId );
    Exit;
  end;
  Result := inherited Same(o);
end;


//------------------------------------------------------------------------------
// TFollowTimeItem
//------------------------------------------------------------------------------
class function TFollowTimeItem.ItemCount : integer;
begin
  Result := 6;
end;

function TFollowTimeItem.GetItem(i:integer):string;
begin
  case i of
    2: Result := DateTimeToStr(DStart);
    3: Result := DateTimeToStr(DStop);
    4: Result := TimeToDiffStr(DDiff);
    else Result := inherited GetItem(i);
  end;  
end;

procedure TFollowTimeItem.SetItem(i:integer; const s:string);
begin
  case i of
    2: DStart := StrToDateTime(s);
    3: DStop := StrToDateTime(s);
    4: ;
    else inherited SetItem(i,s);
  end;  
end;

class function TFollowTimeItem.GetHead(i:integer):string;
begin
  Result := '?';
  case i of
    0: Result := 'MainID';
    1: Result := 'SubID';
    2: Result := 'Start time';
    3: Result := 'Stop time';
    4: Result := 'Diff';
    5: Result := 'Expl';
  end;
end;


procedure TFollowTimeItem.SetSt(const st:TStrings);
begin
  MainId := st[0];
  SubId := st[1];
  Start := st[2];
  Stop := st[3];
//  Diff := st[4];
  Expl := st[5];
end;

procedure TFollowTimeItem.GetSt(st:TStrings);
begin
  st.Add(MainId);
  st.Add(SubId);
  st.Add(Start);
  st.Add(Stop);
  st.Add(Diff);
  st.Add(Expl);
end;

const eps = -0.000000001;

function TFollowTimeItem.Compare(o:TFileObject;i:integer):integer;
var d:double; ti : TFollowTimeItem;
begin
  if not (o is TFollowTimeItem) then begin Result := inherited compare(o,i); exit; end;
  ti := o as TFollowTimeItem;
  case i of
    2: d := DStart - ti.DStart;
    3: d := DStop - ti.DStop;
    4: d := DDiff - ti.DDiff;
    else d := inherited Compare(o,i);
  end;
  Result := -0;
  if d < -eps then Result := -1;
  if d > eps  then Result := 1;
end;

procedure TFollowTimeItem.GetId(const prj:TFollowProjectItem);
begin
  MainId := prj.MainId;
  SubId := prj.SubId;
end;

procedure TFollowTimeItem.DoStart;
begin
  DStart := Now;
end;

procedure TFollowTimeItem.DoStop;
begin
  DStop := Now;
end;

procedure TFollowTimeItem.SetDStop(d:TDateTime);
begin
  FDStop := d;
  FStop := FloatToStr(DStop);
  DDiff := DStop-DStart;
end;

procedure TFollowTimeItem.SetDStart(d:TDateTime);
begin
  FDStart := d;
  FStart := FloatToStr(DStart);
  DDiff := DStop-DStart;
end;

procedure TFollowTimeItem.SetDDiff(d:TDateTime);
begin
  FDDiff := d;
  FDiff := Format('%1.2f',[(DDiff)*60*24]);
end;

procedure TFollowTimeItem.SetStart(const s:string);
begin
  DStart := StrToFloatDef(s,0);
end;

procedure TFollowTimeItem.SetStop(const s:string);
begin
  DStop := StrToFloatDef(s,0);
end;


procedure TFollowTimeItem.DoStopExpl(const exp:string);
begin
  DoStop;
  Expl := exp;
end;

function TFollowTimeItem.Same(o:TFileObject):boolean;
var pi : TFollowProjectItem;
begin
  if ( o is TFollowProjectItem ) then begin
    pi := o as TFollowProjectItem;
    Result := ( pi.MainId = MainId ) and ( pi.SubId = SubId );
    Exit;
  end;
  Result := inherited Same(o);
end;

end.
