unit timefoll;
{
  Program for controlling time usage for different projects.

  Author:  Vesa Lappalainen
  Date:    26.12.1997
  Changes: 27.12.1997/vl
   + explanation field to time file
   + save start time to temp file
   + in FormCreate look for temp file
   + options dialog for filenames etc.
   + add new time by hand
  Changes: 28.12.1997/vl
   + add new project and edit project
   + give date thru clendar
   + auto start/stop when Project change
   + combobox align client
   + show dates in diff
  Changes: 30.09.2001/vl
   + VLC/CLX-compiling

  Missing:
   - option for diff format in file
   - history ComboBox -component
   - option for tmp save interval
   - count used times
   - convert to database

  Usage:
    Contains files:
      .pro - Projects file - Project id's and explanation
      .tim - Time file     - Project id + start + stop + diff + explanation
      .tmp - Temp file     - unstoppped time
      .ini                 - options

  Installation:
    Copy .exe, .pro, .ini, .time to some directory.

    If need for personal .ini file, copy .ini to personal file and start
    TimeFollow in that directory.

  Compilation:
    Needs following files
     timefoll.pas    - this file
     timefollow.prj  - delphi project file
     fileobj.pas     - clasess for handling textfile objects
     follitem.pas    - clasess inherited to this programs purposes
     filecomb.pas    - File shown with combobox

     kString.pas     - Kave's string handling routines
     kDouble.pas     - Kave's double handling routines
     AskDate.pas     - Kave's Date asking dialog

    Following components must be installed before compilation:
     kIniComp.pas    - components that automatically stores themself to .ini
     SavePos.pas     - component that stores form size and pos.
                       e:\kave\k2000\kcomp
}

interface

uses
  SysUtils, Classes,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs,
  QMenus, QStdCtrls, QComCtrls, QExtCtrls, QGrids, QTypes,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls, ExtCtrls, Grids,
{$endif}
  IniReadOrder,fileobj,savepos, kinicomp,follitem,filecomb;

//const Version:string = 'Version 1.0 / 28.12.1997';
const Version:string = 'Version 1.1 / 30.09.2001';

type
  TFormFollow = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Options1: TMenuItem;
    QApplicationoptions1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Help2: TMenuItem;
    PanelControls: TPanel;
    StatusBar1: TStatusBar;
    SavePos1: TSavePos;
    Timer: TTimer;
    PanelExpl: TPanel;
    PanelButtons: TPanel;
    TimerTmp: TTimer;
    Edit1: TMenuItem;
    AddManually1: TMenuItem;
    Insertproject1: TMenuItem;
    Editproject1: TMenuItem;
    PanelStartStop: TPanel;
    ButtonStop: TButton;
    ButtonStart: TButton;
    CBProjects: TComboBox;
    PanelSeparator: TPanel;
    EditExpl: TEditIni;
    Edittimes1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure CBProjectsChange(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure QApplicationoptions1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTmpTimer(Sender: TObject);
    procedure AddManually1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure About1Click(Sender: TObject);
    procedure Insertproject1Click(Sender: TObject);
    procedure Editproject1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Edittimes1Click(Sender: TObject);
  private
    { Private declarations }
    projects : TFileCombo;
    thistime : TFollowTimeItem;
    times : TFileObjects;
    tmptime : TFileObjects;
    Running : boolean;
    CaptionStr : string;
    RunId : integer;
    procedure ShowTime;
    procedure DoStopEnable(b:boolean);
    procedure SaveTmp;
    procedure DoStart;
    procedure DoContinue;
    procedure DoFullStart;
    procedure DoStop;
    function DoFullStop:boolean;
    function DoEdit(const s:string;cont:boolean=false):boolean;
  public
    { Public declarations }
    procedure InitForm;
    procedure ClearForm;
  end;

var
  FormFollow: TFormFollow;

implementation

uses foptions, edittime, FolAbout, editprj,timeview, kstring, kdouble;

{$ifdef CLX}
{$R *.xfm}
{$else}
{$R *.dfm}
{$endif}

procedure DeleteFile2(const s:string);
begin
{$ifdef CLX}
  DeleteFile(s);
{$else}
  DeleteFile(PChar(s));
{$endif}
end;

//------------------------------------------------------------------------------
// TFormFollow
//------------------------------------------------------------------------------

procedure TFormFollow.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TFormFollow.ClearForm;
begin
  if ( projects <> nil ) then projects.Free;  projects := nil;
  if ( thistime <> nil ) then thistime.Free;  thistime := nil;
//  Options.Ini.WriteInteger('FormFollow-Params','CBProjectsItemIndex',
//                           CBProjects.ItemIndex);
  if ( times    <> nil ) then times.Free;     times    := nil;
  if ( tmptime  <> nil ) then tmptime.Free;   tmptime  := nil;

end;

procedure TFormFollow.DoStart;
begin
  thistime.GetId(projects.Obj[CBProjects.ItemIndex] as TFollowProjectItem);
  thistime.DoStart;
  hint := projects.Obj[CBProjects.ItemIndex].HintStr;
end;

procedure TFormFollow.DoStop;
begin
  thistime.DoStopExpl(EditExpl.Text);
end;


procedure TFormFollow.InitForm;
var i:integer;
begin
  LongTimeFormat := 'HH:mm:ss';
  TimeSeparator := ':';
  ShortDateFormat := 'yyyy.mm.dd';
  DateSeparator := '.';
  ClearForm;
  projects := TFileCombo.Create3(TFollowProjectItem,Options.ProjectName.Text,CBProjects);
  thistime := TFollowTimeItem.Create('');
  projects.ReadFromFile('');
                      
//  i := Options.Ini.ReadInteger('FormFollow-Params','CBProjectsItemIndex',0);
  i := IniOrderReadInteger('','FormFollow-Params','CBProjectsItemIndex',0);
  if ( i < 0 ) then i := 0;
  CBProjects.ItemIndex := i;

  DoStart;
  DoStop;
  times := TFileObjects.Create2(TFollowTimeItem,Options.TimeName.Text);
  tmptime := TFileObjects.Create2(TFollowTimeItem,Options.TempName.Text);
  tmptime.ReadFromFile('');
  if ( tmptime.Count > 0 ) then begin
    thistime.Str := tmptime.Obj[0].Str;
    if ( DoEdit('Previous project not stopped! Add!') ) then
      times.AppendToFile(thistime,'');
    DeleteFile2(Options.TempName.Text);
  end;
end;

procedure TFormFollow.CBProjectsChange(Sender: TObject);
begin
  if ( Options.AutoProj.Checked and Running ) then
    if ( RunID = CBProjects.ItemIndex ) then exit
    else if DoFullStop then DoFullStart;
  if ( not Running ) then begin
    hint := projects.Obj[CBProjects.ItemIndex].HintStr;
    StatusBar1.Panels[0].Text := Hint;
  end;
end;

procedure TFormFollow.DoStopEnable(b:boolean);
begin
  Running := b;
  ButtonStop.Visible := b;
  ButtonStart.Visible := not b;
  CBProjects.Enabled := (not b) or Options.AutoProj.Checked;
  QApplicationoptions1.Enabled := not b;
  AddManually1.Enabled := not b;
  Timer.Enabled := b;
  TimerTmp.Enabled := b;
  DeleteFile2(Options.TempName.Text);
end;

function TFormFollow.DoFullStop:boolean;
begin
  Result := false;
  DoStop;
  DoStopEnable(false);
  StatusBar1.Panels[1].Text := StatusBar1.Panels[0].Text;
  Caption := CaptionStr;   
  if ( Options.VerifyAdd.Checked ) and
     ( not ( DoEdit('OK to add time?',true) )) then  exit;
  times.AppendToFile(thistime,'');
  Result := true;
end;

procedure TFormFollow.DoContinue;
var o : TFollowProjectItem;
begin
  DoStopEnable(true);
  RunId := projects.Find(thistime);
  o := projects.Obj[RunId] as TFollowProjectItem;
  hint := o.HintStr;
  Caption := CaptionStr + ' - ' + o.Expl;
  SaveTmp;
  ShowTime;
end;

procedure TFormFollow.DoFullStart;
begin
  DoStart;
  DoContinue;
end;

procedure TFormFollow.ButtonStopClick(Sender: TObject);
begin
  DoFullStop;
end;

procedure TFormFollow.ButtonStartClick(Sender: TObject);
begin
  DoFullStart;
end;

procedure TFormFollow.ShowTime;
begin
  thistime.DoStop;
  StatusBar1.Panels[0].Text := TimeToStr(thistime.DDiff);
end;

procedure TFormFollow.TimerTimer(Sender: TObject);
begin
  ShowTime;
end;

procedure TFormFollow.QApplicationoptions1Click(Sender: TObject);
begin
  Options.ShowModal;
  InitForm;
end;


procedure TFormFollow.FormDestroy(Sender: TObject);
begin
  ClearForm;
end;

procedure TFormFollow.SaveTmp;
begin
  DoStop;
  tmptime.Clear;
  tmptime.AddS(thistime.Str);
  tmptime.WriteToFile('');
end;

procedure TFormFollow.TimerTmpTimer(Sender: TObject);
begin
  SaveTmp;
end;

procedure TFormFollow.AddManually1Click(Sender: TObject);
begin
  if ( DoEdit('Add new item') ) then times.AppendToFile(thistime,'');
end;

function TFormFollow.DoEdit(const s:string;cont:boolean) : boolean;
var mr : integer;
begin
  Result := false;
  mr := EditTimeItem.Edit(projects,thistime,s,cont);
  if ( mr = mrRetry ) then begin DoContinue;  Exit; end;
  Result := ( mr = mrOK );
end;

procedure TFormFollow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  IniOrderWriteInteger('','FormFollow-Params',
    'CBProjectsItemIndex',CBProjects.ItemIndex);
  if ( Running ) and ( Options.AutoStop.Checked ) then CanClose := DoFullStop;
end;

procedure TFormFollow.About1Click(Sender: TObject);
begin
  AboutBox.ShowAbout(CaptionStr,Version);
end;

procedure TFormFollow.Insertproject1Click(Sender: TObject);
var item : TFollowProjectItem;
begin
  item := projects.Item as TFollowProjectItem;
  item.Str := projects.Obj[CBProjects.ItemIndex].Str;
  if ( EditProject.Edit(item,'Insert new project') <> mrOK ) then exit;
  projects.AddS(item.Str);
  projects.WriteToFile('');
end;

procedure TFormFollow.Editproject1Click(Sender: TObject);
var item : TFollowProjectItem;
begin
  item := projects.Obj[CBProjects.ItemIndex] as TFollowProjectItem;
  if ( EditProject.Edit(item,'Edit project') <> mrOK ) then exit;
  projects.Update;
  projects.WriteToFile('');
end;

procedure TFormFollow.FormCreate(Sender: TObject);
begin
//  SetNumberFormat;
  CBProjects.Align := alClient;
  ButtonStop.Align := alClient;
  ButtonStart.Align := alClient;
//  CaptionStr := Caption;
{$ifdef CLX}
  CaptionStr := 'TimeFollow for CLX';
{$else}
  CaptionStr := 'TimeFollow for Win95/NT';
{$endif}
  Caption := CaptionStr;
end;

procedure TFormFollow.Edittimes1Click(Sender: TObject);
begin
  ViewTimes(projects,Options.TimeName.Text);
end;

end.
