{-----------------------------------------------------------------------------
   Unit Name: timeview
   Purpose  : View time in list
   Author   : Vesa Lappalainen
   Date     : 26.12.1997
   Changed  :
   Todo     :
-----------------------------------------------------------------------------}

unit timeview;

interface

uses
  SysUtils, Classes,
{$ifdef CLX}
  QTypes, QGraphics, QControls, QForms, QDialogs, QComCtrls, QExtCtrls, QMenus,
  QStdCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, ToolWin, ComCtrls, ExtCtrls, Menus,
  StdCtrls,
{$endif}
  filecomb,filelist, follitem, savepos;


type
  TFormTimeView = class(TForm)
    MainMenu1: TMainMenu;
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    ToolBar1: TToolBar;
    ListView: TListView;
    ButtonClose: TButton;
    SavePos1: TSavePos;
    ButtonSave: TButton;
    procedure ButtonCloseClick(Sender: TObject);
    procedure ListViewDblClick(Sender: TObject);
    function DoEdit(const s:string; thisitem:TFollowTimeItem) : boolean;
    procedure ButtonSaveClick(Sender: TObject);
  private
    { Private declarations }
    times : TFileList;
    projects : TFileCombo;
  public
    { Public declarations }
    procedure Save;
  end;
  function ViewTimes(prj:TFileCombo;const filename:string):integer;

var
  FormTimeView: TFormTimeView;

implementation

uses edittime,IniReadOrder;
{$ifdef CLX}
{$R *.xfm}
{$else}
{$R *.dfm}
{$endif}

procedure ReadColumns(li:TListView;const s:string);
var i:integer;
begin
  for i:=0 to li.Columns.Count-1 do
    li.Columns[i].Width := IniOrderReadInteger('',s,'ColW'+IntToStr(i),li.Columns[i].Width);
end;

procedure SaveColumns(li:TListView; const s:string);
var i:integer;
begin
  for i:=0 to li.Columns.Count-1 do
    IniOrderWriteInteger('',s,'ColW'+IntToStr(i),li.Columns[i].Width);
end;

function ViewTimes(prj:TFileCombo;const filename:string):integer;
var ftv : TFormTimeView;
begin
  ftv := TFormTimeView.Create(nil);
  ftv.times := TFileList.Create3(TFollowTimeItem,filename,ftv.ListView);
  ftv.times.ReadFromFile('');
  ftv.projects := prj;
  ReadColumns(ftv.ListView,'ViewTimesList');
  Result := ftv.ShowModal;
  SaveColumns(ftv.ListView,'ViewTimesList');
  ftv.times.free;
  ftv.Free;
end;

procedure TFormTimeView.ButtonCloseClick(Sender: TObject);
var mr : word;
begin
  if ( ButtonSave.Enabled ) then begin
    mr := MessageDlg('Save file?'+#13#10+times.Name,
                     mtConfirmation,[mbYes,mbNo,mbCancel],0);
    if ( mr = mrCancel ) then exit;
    if ( mr = mrOK ) then times.WriteToFile('');
  end;
  Close;
end;

function TFormTimeView.DoEdit(const s:string; thisitem:TFollowTimeItem) : boolean;
var mr : integer;
begin
  mr := EditTimeItem.Edit(projects,thisitem,s);
  Result := ( mr = mrOK );
end;

procedure TFormTimeView.ListViewDblClick(Sender: TObject);
var thisitem : TFollowTimeItem;
begin
  if ( ListView.Selected = nil ) then exit;
  thisitem := (ListView.Selected.Data);
  if ( DoEdit('Edit item',thisitem) ) then begin
    ButtonSave.Enabled := true; 
    times.UpdateLine(ListView.Selected);
  end;
//
end;

procedure TFormTimeView.Save;
begin
  times.WriteToFile('');
  ButtonSave.Enabled := false;
end;

procedure TFormTimeView.ButtonSaveClick(Sender: TObject);
begin
  Save;
end;

end.
